﻿using Windows.ApplicationModel.LockScreen;
using Windows.ApplicationModel.Preview.Holographic;
using Windows.Foundation.Metadata;
using Windows.System.Profile;
using Windows.UI.ViewManagement;

namespace TestProject
{
    /// <summary>
    /// 윈도우 디스플레이 정보
    /// </summary>
    public sealed class WindowDisplayInformation
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 현재 윈도우 디스플레이 모드 구하기 - GetCurrentWindowDisplayMode()

        /// <summary>
        /// 현재 윈도우 디스플레이 모드 구하기
        /// </summary>
        /// <returns>현재 윈도우 디스플레이 모드</returns>
        public static WindowDisplayMode GetCurrentWindowDisplayMode()
        {
            ApplicationView applicationView = ApplicationView.GetForCurrentView();

            switch(AnalyticsInfo.VersionInfo.DeviceFamily)
            {
                case "Windows.Desktop" :
                {
                    if
                    (
                        ApiInformation.IsEnumNamedValuePresent("Windows.UI.ViewManagement.ApplicationViewMode", "CompactOverlay") &&
                        applicationView.ViewMode == ApplicationViewMode.CompactOverlay
                    )
                    {
                        return WindowDisplayMode.CompactOverlay;
                    }

                    if(LockApplicationHost.GetForCurrentView() != null)
                    {
                        return WindowDisplayMode.FullScreen;
                    }

                    if(IsMixedReality())
                    {
                        return WindowDisplayMode.Windowed;
                    }
                    else
                    {
                        if(applicationView.IsFullScreenMode)
                        {
                            return WindowDisplayMode.FullScreen;
                        }
                        else
                        {
                            switch(UIViewSettings.GetForCurrentView().UserInteractionMode)
                            {
                                case UserInteractionMode.Mouse :
#pragma warning disable CS0618
                                    return applicationView.IsFullScreen ? WindowDisplayMode.Maximized : WindowDisplayMode.Windowed;
#pragma warning restore CS0618

                                case UserInteractionMode.Touch :
                                {
                                    if(applicationView.AdjacentToLeftDisplayEdge)
                                    {
                                        if(applicationView.AdjacentToRightDisplayEdge)
                                        {
                                            return WindowDisplayMode.FullScreenTabletMode;
                                        }
                                        else
                                        {
                                            return WindowDisplayMode.SnappedLeft;
                                        }
                                    }
                                    else
                                    {
                                        return WindowDisplayMode.SnappedRight;
                                    }
                                }

                                default :
 
                                    return WindowDisplayMode.Unknown;
                            }
                        }
                    }
                }
                case "Windows.Mobile" :
                {
                    if(UIViewSettings.GetForCurrentView().UserInteractionMode == UserInteractionMode.Mouse)
                    {
                        return applicationView.IsFullScreenMode ? WindowDisplayMode.Maximized : WindowDisplayMode.Windowed;
                    }
                    else
                    {
                        return WindowDisplayMode.FullScreen;
                    }
                }
                case "Windows.Holographic" :
                {
                    return WindowDisplayMode.Windowed;
                }
                case "Windows.Xbox" :
                case "Windows.IoT"  :
                {
                    return WindowDisplayMode.FullScreen;
                }
                case "Windows.Team" :
                {
                    if(applicationView.AdjacentToLeftDisplayEdge)
                    {
                        if(applicationView.AdjacentToRightDisplayEdge)
                        {
                            return WindowDisplayMode.FullScreenTabletMode;
                        }
                        else
                        {
                            return WindowDisplayMode.SnappedLeft;
                        }
                    }
                    else
                    {
                        return WindowDisplayMode.SnappedRight;
                    }
                }
                default :
                {
                    return WindowDisplayMode.Unknown;
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 혼합 현실 여부 구하기 - IsMixedReality()

        /// <summary>
        /// 혼합 현실 여부 구하기
        /// </summary>
        /// <returns>혼합 현실 여부</returns>
        private static bool IsMixedReality()
        {
            try
            {
                bool result = ApiInformation.IsTypePresent("Windows.ApplicationModel.Preview.Holographic.HolographicApplicationPreview") &&
                              ApiInformation.IsMethodPresent
                              (
                                  "Windows.ApplicationModel.Preview.Holographic.HolographicApplicationPreview",
                                  nameof(HolographicApplicationPreview.IsCurrentViewPresentedOnHolographicDisplay)
                              ) &&
                              HolographicApplicationPreview.IsCurrentViewPresentedOnHolographicDisplay();

                return result;
            }
            catch
            {
            }

            return false;
        }

        #endregion
    }
}